<?php

namespace Litecms\Article\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litecms\Article\Forms\Article as ArticleForm;
use Litecms\Article\Http\Requests\ArticleRequest;
use Litecms\Article\Interfaces\ArticleRepositoryInterface;
use Litecms\Article\Repositories\Eloquent\Filters\ArticleResourceFilter;
use Litecms\Article\Repositories\Eloquent\Presenters\ArticleListPresenter;

/**
 * Resource controller class for article.
 */
class ArticleResourceController extends BaseController
{

    /**
     * Initialize article resource controller.
     *
     *
     * @return null
     */
    public function __construct(ArticleRepositoryInterface $article)
    {
        parent::__construct();
        $this->form = ArticleForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litecms.article.modules'), 'article', guard_url('article'));
        $this->repository = $article;
    }

    /**
     * Display a list of article.
     *
     * @return Response
     */
    public function index(ArticleRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(ArticleResourceFilter::class)
            ->setPresenter(ArticleListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('article::article.names'))
            ->view('article::article.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display article.
     *
     * @param Request $request
     * @param Model   $article
     *
     * @return Response
     */
    public function show(ArticleRequest $request, ArticleRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('article::article.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('article::article.show')
            ->output();
    }

    /**
     * Show the form for creating a new article.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(ArticleRequest $request, ArticleRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('article::article.name'))
            ->view('article::article.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new article.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ArticleRequest $request, ArticleRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('article::article.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('article/article/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/article/article'))
                ->redirect();
        }

    }

    /**
     * Show article for editing.
     *
     * @param Request $request
     * @param Model   $article
     *
     * @return Response
     */
    public function edit(ArticleRequest $request, ArticleRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('article::article.name'))
            ->view('article::article.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the article.
     *
     * @param Request $request
     * @param Model   $article
     *
     * @return Response
     */
    public function update(ArticleRequest $request, ArticleRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('article::article.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('article/article/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('article/article/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the article.
     *
     * @param Model   $article
     *
     * @return Response
     */
    public function destroy(ArticleRequest $request, ArticleRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('article::article.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('article/article/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('article/article/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
